// CONIO_TC.h
//
// Old Turbo-C CONIO.H compatibility library for LCC-Win32
// and GCC/EGCS Mingw32 compilers.
// Version 1.0 (September 1999).
// Created by Daniel Guerrero Miralles (daguer@geocities.com).
// This source code is public domain.
//
// v2.00, Jason Hood, 7 to 9 June, 2005:
//   turned into a library (for MinGW32);
//   use buffer width/window height on program startup as screen size;
//   working window function, but text output is not clipped.
//
// Todo: custom output functions, to recognise the window;
//	 textmode could be made to work (incorporate Console Size?);
//	 full screen applications won't work near the end of the buffer
//	  until textmode or clrscr is used;
//	 should probably be using _UNICODE, wchar_t & *wtext*.
//
// v2.01 Greg Spears, Oct 2014 -- Added some functions, changed (improved?)  
//   some of the coding. Also folded in some of James Daughtry's "conio.h 
//   Library Replacement" code into this library.  Hence, almost everything
//   you really need of the old Turbo C CONIO functions is here, now
//   ported to Win32 code.  The few missing are documented at the
//   end of CONIO_TC.h -- you won't miss them, I think.
//   Most notably missing is cscanf(), I've never used it, and
//   is possibly horrific to code.
//

#ifdef _INC_CONIO
  /*It shouldn't cause any real harm, but lets caution on it. User can comment out this line if desired*/
  #error conio.h is not compatible with this library!
#endif //_INC_CONIO


#ifndef _CONIO_TC_H_
#define _CONIO_TC_H_

// avoid including windows.h just for HANDLE & WCHAR
typedef void* HANDLE;
typedef unsigned short WCHAR;


#if 0
	/* Function prototypes where we still need MSVC conio.h or equiv here */
	_CRTIMP int __cdecl _cscanf(const char *, ...);
#endif  //0

extern int directvideo; // useless, only for compatibility.
extern int _wscroll;	// Partially implemented

extern HANDLE hConOut, hConIn;	// console output and input handles


// CONTROL FUNCTIONS //////////////////////////////////////////////////////

// get some information about the console state
struct text_info
{
  unsigned char winleft;
  unsigned char wintop;
  unsigned char winright;
  unsigned char winbottom;
  unsigned char attribute;
  unsigned char normattr;
  unsigned char currmode;
  unsigned char screenheight;
  unsigned char screenwidth;
  unsigned char curx;
  unsigned char cury;
};

void gettextinfo( struct text_info* pTI );

// set the video mode
enum text_modes
{
  LASTMODE = -1, /*The text mode which was in effect before the last call to textmode().*/
  BW40,			/* 40-column black and white (on a color screen)*/
  C40,			/* 40-color color.*/
  BW80,			/* 80-column black and white (on a color screen)*/
  C80,			/* 80-column color  (assume 25 rows)*/
  MONO = 7,		/* The monochrome monitor*/
  C4350 = 64	/* 80x50 color. 80-column, 43-rows(on EGAs) 50-rows(on VGAs) color*/
};

// NOTE: This function does nothing but reset the console
// to default state.
void textmode( int newmode );

// change the size and position of the text i/o window
// IMPORTANT NOTE: text output functions are not clipped
void window( int left, int top, int right, int bottom );


// CURSOR FUNCTIONS ///////////////////////////////////////////////////////
// NOTE: Cursor coordinates begin at (1,1) istead of (0,0)

// get cursor horizontal position
int wherex( void );

// get cursor vertical position
int wherey( void );

// change the cursor location
void gotoxy( int x, int y );

// set the cursor shape
#define _NOCURSOR	0
#define _SOLIDCURSOR	1
#define _NORMALCURSOR	2
void _setcursortype( int cur_t );


// COLOR FUNCTIONS ////////////////////////////////////////////////////////

// change text color to high intensity
void highvideo( void );

// change text color to low intensity
void lowvideo( void );

// set default text colors
void normvideo( void );

// set the text foreground/background colors
#if !defined(__COLORS)
#define __COLORS
enum COLORS {
    BLACK,         // 0         /* dark colors */
    BLUE,          // 1
    GREEN,         // 2
    CYAN,          // 3
    RED,           // 4
    MAGENTA,       // 5
    BROWN,         // 6
    LIGHTGRAY,     // 7
    DARKGRAY,      // 8         /* light colors */
    LIGHTBLUE,     // 9
    LIGHTGREEN,    // 10
    LIGHTCYAN,     // 11
    LIGHTRED,      // 12
    LIGHTMAGENTA,  // 13
    YELLOW,        // 14
    WHITE          // 15
};
#endif //__COLORS

#define BLINK	128	// useless, background always high intensity.

void textattr( int newattr );

// set the text background color
void textbackground( int newcolor );

// set the text color
void textcolor( int newcolor );


// TEXT I/O FUNCTIONS /////////////////////////////////////////////////////

// clear up to end-of-line
void clreol( void );

// clear the screen
void clrscr( void );

// insert a line
void insline( void );

// delete current line
void delline( void );

// copy a rectangular section of the screen
int movetext( int left, int top, int right, int bottom,
	      int destleft, int desttop );

// get text from a rectangular section of the screen
// IMPORTANT NOTE: In the ASCII version of gettext() and puttext(), both the
// character and the attribute must be of type 'char' (8 bits), while in the
// UNICODE version both must be of the type WCHAR (16 bits).
int gettextA( int left, int top, int right, int bottom, void* destin );
int gettextW( int left, int top, int right, int bottom, void* destin );

// set text in a rectangular section of the screen
int puttextA( int left, int top, int right, int bottom, void* source );
int puttextW( int left, int top, int right, int bottom, void* source );

// read a password from keyboard
char*  getpassA( const char*  prompt );
WCHAR* getpassW( const WCHAR* prompt );

#ifdef UNICODE
#define gettext gettextW
#define puttext puttextW
#define getpass getpassW
#else // !UNICODE
#define gettext gettextA
#define puttext puttextA
#define getpass getpassA
#endif // !UNICODE


/*---------Add ons--------------*/
//Like putch() put a character to the console
int putch_tc(int ch);

//see the kybd was hit
int kbhit_tc(void);

//getch() - get a key
int getch_tc(void);

//getche() - get a key and echo it
int getche_tc(void);

// ungetch() - put a key back into the buffer
int ungetch_tc(int ch);

// cputs() - puta string to the console
int cputs_tc(const char *instr);

// cprintf() - put a formatted string to the console
int cprintf_tc( const char *fmt, ... );

//cgets() read a string from the console input
char *cgets_tc( char *inbuffer );

/*- Of course, we must undef and re-define the prior definitions.
* But then -- they should not be defined -- conio.h 
* should Not be #included
*/

#undef _putch
#undef _kbhit
#undef _getch
#undef _getche
#undef _ungetch
#undef _cputs
#undef _cprintf
#undef _cgets

#undef putch
#undef kbhit
#undef getch
#undef getche
#undef ungetch
#undef cputs
#undef cprintf
#undef cgets

#define _putch putch_tc
#define _kbhit kbhit_tc
#define _getch getch_tc
#define _getche getche_tc
#define _ungetch ungetch_tc
#define _cputs	cputs_tc
#define _cprintf cprintf_tc
#define _cgets	cgets_tc

#define putch putch_tc
#define kbhit kbhit_tc
#define getch getch_tc
#define getche getche_tc
#define ungetch ungetch_tc
#define cputs	cputs_tc
#define cprintf cprintf_tc
#define cgets	cgets_tc


/* All the following are not found in this library .
*  Everything else is present and true-to-form Borland Turbo C substitutes
*/

#if 0 ////////////////////////////////
int			_Cdecl cscanf( const char *__format, ... ); 
int         _Cdecl inp( unsigned __portid );
unsigned    _Cdecl inpw( unsigned __portid );
int         _Cdecl outp( unsigned __portid, int __value );
unsigned    _Cdecl outpw( unsigned __portid, unsigned __value );
unsigned char _Cdecl inportb( int __portid );
void        _Cdecl outportb( int __portid, unsigned char __value );
int         _Cdecl inport( int __portid );
void        _Cdecl outport( int __portid, int __value );
#endif /////////////////////////////////////////////////// 0



#endif // _CONIO_TC_H_